-- creates the bars that display gambits and allow the user to select one for the main WE window
import "Turbine.Gameplay";

WEQuickslotWindow = SDRPlugins.WardenEase2.class( Turbine.UI.Window );

function WEQuickslotWindow:Constructor(builderType, settings)
	Turbine.UI.Window.Constructor( self );

	self.settings = settings;
	self:SetPosition( self.settings.left, self.settings.top );
	self.gambitBuilder = builderType;
	self.spacing = 2;

	self.quickslotSize = 32;
	self.descriptions = { };
	self.gambitControls = { };

	levelChangedHandler2 = function( sender, args )
		self:RefreshQuickslots();
	end

	AddCallback(player, "LevelChanged", levelChangedHandler2);

	self.PositionChanged = function( sender, args )
		self.settings.left, self.settings.top = self:GetPosition();
		self:SaveSettings();
	end
	
	self:RefreshQuickslots();

	self.DragBar = SDRPlugins.WardenEase2.UI.DragBar( self, SDRPlugins.WardenEase2[self.gambitBuilder] );
	self.DragBar:SetRotator( true );
	
	self:SetVisible( true );
	self:SetOpacity( 1 );
end

-- switches the bar from horizontal to vertical and back
function WEQuickslotWindow:Rotate( args )
	self.settings.transposed = not self.settings.transposed;
	self:SaveSettings();
	self:RefreshQuickslots();
	self.DragBar:RecalculateSize();
end

function WEQuickslotWindow:SaveSettings()
	SDRPlugins.WardenEase2.WEmain:SaveSettings();
end

-- refreshes quickslots to add new gambits or rotate them when needed
function WEQuickslotWindow:RefreshQuickslots()
	local i, k, v;
	local descriptionsOn = self.settings.descriptionsOn;

	self.gambits = player:GetClassAttributes():GetTrainedGambits();
	self.barGambits = {}

	-- using the known gambit list, create the barGambits
	for i = 1, self.gambits:GetCount() do
		local skillInfo = self.gambits:GetItem(i):GetSkillInfo();
		--Turbine.Shell.WriteLine("Gambit Count: " .. skillInfo:GetGambitCount());
		--Turbine.Shell.WriteLine("Description: " .. skillInfo:GetDescription());

		if skillInfo:GetDescription():find("\n") then
			--Turbine.Shell.WriteLine("Found new line character");
		end

		local skillName = skillInfo:GetName(); -- revert to above once API is fixed
		local GambitTable = SDRPlugins.WardenEase2.Gambits;

		local containsJavelinBuilder = false; -- we don't want gambits containing Javelin skills because they are just duplicates
		--for i = 1, skillInfo:GetGambitCount() do
			--if skillInfo:GetGambit(i) == 4 then
				--containsJavelinBuilder = true;
			--end
		--end

		-- Post-HD fix (revert to above once API is fixed)
		if GambitTable[skillName] ~= nil then
			for i = 1,  #GambitTable[skillName] do
				if GambitTable[skillName][i] == 4 then
					containsJavelinBuilder = true;
				end
			end

			if not containsJavelinBuilder and GambitTable[skillName][1] == SDRPlugins.WardenEase2.WardenGambits[self.gambitBuilder] and self.settings.hiddenGambits[skillInfo:GetName()] == nil then
				--Turbine.Shell.WriteLine(skillName .. " - " .. SDRPlugins.WardenEase2.WardenGambits[self.gambitBuilder]);
				local spotToAdd = 1;
				local addGambit = true;
				for index = 1, #self.barGambits do
					if #GambitTable[self.barGambits[spotToAdd]:GetName()] == #GambitTable[skillName] then
						if self.barGambits[spotToAdd]:GetName() == skillName then
							addGambit = false;
							break;
						end
					elseif #GambitTable[self.barGambits[spotToAdd]:GetName()] > #GambitTable[skillName] then
						break;
					end
					spotToAdd = spotToAdd + 1;
				end
				if addGambit then
					table.insert(self.barGambits, spotToAdd, skillInfo);
				end
			end
		else
			Turbine.Shell.WriteLine(skillName); -- prints out gambits that aren't in the table
		end
		
		-- checks to see if the first builder matches the bar builder type (and makes sure that the gambit doesn't include javelin skills)
		-- uncomment this out once API is fixed
		-- 	also checks to make sure it is not a hidden gambit
		--if not containsJavelinBuilder and skillInfo:GetGambit(1) == SDRPlugins.WardenEase2.WardenGambits[self.gambitBuilder] and self.settings.hiddenGambits[skillInfo:GetName()] == nil then
			--table.insert(self.barGambits, skillInfo);
		--end
		
	end

	-- Remove any quickslots no longer required.
	for i = #(self.barGambits) + 1, #(self.gambitControls), 1 do
		-- remove the gambits from the gambits control (gambit controls contain the actual picture of the gambit the user clicks)
		self.gambitControls[#(self.barGambits) + 1]:SetVisible( false );
		self.gambitControls[#(self.barGambits) + 1]:SetParent( nil );
		table.remove(self.gambitControls, #(self.barGambits) + 1);
	end
	
	-- Add any new quickslots that should be created.
	-- creates the actual control containing the gambit the user clicks
	for i = #(self.gambitControls) + 1, #(self.barGambits), 1 do
		self.gambitControls[i] = Turbine.UI.Control();
		self.gambitControls[i]:SetSize( self.quickslotSize, self.quickslotSize );
		self.gambitControls[i]:SetParent( self);
		self.gambitControls[i]:SetVisible(true);
		self.gambitControls[i]:SetAllowDrop( false );
		self.gambitControls[i]:SetBackground(self.barGambits[i]:GetIconImageID());
		self.gambitControls[i].gambit = self.barGambits[i];

		-- sets the gambit when the user clicks the icon
		self.gambitControls[i].MouseClick = function( sender, args )
			if args.Button == Turbine.UI.MouseButton.Left then
				SDRPlugins.WardenEase2.WEmain:SetGambit( sender.gambit );
			elseif args.Button == Turbine.UI.MouseButton.Right then

				local contextMenu = Turbine.UI.ContextMenu()
				local contextMenuItems = contextMenu:GetItems();

				-- create the "hide" option
				local subMenu = Turbine.UI.MenuItem(SDRPlugins.WardenEase2.hide ..sender.gambit:GetName());
				subMenu.Click = function( innerSender, args )
					if #(self.gambitControls) == 1 then
						Turbine.Shell.WriteLine(SDRPlugins.WardenEase2.hideWholeBarErrorMessage);
					else
						self.settings.hiddenGambits[sender.gambit:GetName()] = false;
						self.gambitControls = {};
						self:RefreshQuickslots();
						self:SaveSettings();
					end
				end
				contextMenuItems:Add( subMenu );

				numOfHiddenGambits = 0; -- allows me to show the "Show all gambits" only when needed
				-- creates the individual submenu parts for any hidden gambits
				for k,v in pairs( self.settings.hiddenGambits ) do
					numOfHiddenGambits = numOfHiddenGambits + 1;
					-- labels the submenu part based on Spear/First/Shield
					local subMenu = Turbine.UI.MenuItem(SDRPlugins.WardenEase2.show .. k );

					-- handler for when the user selects one of the submenu parts
					subMenu.Click = function( sender, args )

						-- re-enable a gambit
						self.settings.hiddenGambits[k] = nil;
						self.gambitControls = {};
						self:RefreshQuickslots();

						-- save settings
						self:SaveSettings();
					end

					-- adds the context sub menu items to the menu
					contextMenuItems:Add( subMenu );
				end

				-- creates a "Show all" button
				if numOfHiddenGambits > 0 then
					-- create a "separator" between the hide gambit option and the hidden gambits
					subMenu = Turbine.UI.MenuItem("---------------------------------------");
					contextMenuItems:Insert( 2, subMenu );

					subMenu = Turbine.UI.MenuItem(SDRPlugins.WardenEase2.showAllHiddenGambits);
					subMenu.Click = function( innerSender, args )
						for k,v in pairs( self.settings.hiddenGambits ) do
							self.settings.hiddenGambits[k] = nil;
						end
						self.gambitControls = {};
						self:RefreshQuickslots();

						-- save settings
						self:SaveSettings();
					end
					contextMenuItems:Add( subMenu );

					-- create a "separator" between the show hidden gambit and show all hidden gambits
					subMenu = Turbine.UI.MenuItem("---------------------------------------");
					contextMenuItems:Insert( contextMenuItems:GetCount(), subMenu );
				end

				-- shows the context menu when the down arrow is clicked
				contextMenu:ShowMenu();
			end
		end

		-- creates the hint for the gambit
		self.gambitControls[i].MouseHover = function( sender, args )
			if self.settings.descriptionsOn and self.gambitControls[i].tooltip == nil then
				self.gambitControls[i].tooltip = self:CreateHint(sender.gambit, sender);
			end
		end

		-- when the mouse leaves the quickslot, make the window nil to close the tooltip
		self.gambitControls[i].MouseLeave = function( sender, args )
			if self.gambitControls[i].tooltip ~= nil then
				self.gambitControls[i].tooltip:Close();
				self.gambitControls[i].tooltip = nil;
			end
		end
	end

	self:RefreshLayout();
end

function AddNewLineCharactersToDescription(description)
	wordTable = {};
	subStringDesc = description;
	while subStringDesc:len() > 0 do
		spaceIndex = subStringDesc:find(" ");
		newLineIndex = subStringDesc:find("\n");
		if spaceIndex ~= nil and (newLineIndex == nil or (newLineIndex ~= nil and spaceIndex < newLineIndex)) then
			tempString = subStringDesc:sub(1, spaceIndex-1);
			table.insert(wordTable, tempString);
			subStringDesc = subStringDesc:sub(spaceIndex+1);
		elseif spaceIndex ~= nil and newLineIndex ~= nil and spaceIndex > newLineIndex then
			tempString = subStringDesc:sub(1, newLineIndex);
			table.insert(wordTable, tempString);
			subStringDesc = subStringDesc:sub(newLineIndex+1);
		elseif spaceIndex == nil then -- add last word to table
			table.insert(wordTable, subStringDesc);
			subStringDesc = "";
		end
	end

	returnDescription = "";
	maxCharactersPerLine = 30;
	while #wordTable > 0 do
		subDescription = wordTable[1];
		table.remove(wordTable, 1);
		while subDescription:len() < maxCharactersPerLine do
			if subDescription:find("\n") ~= nil then
				returnDescription = returnDescription .. subDescription;
				break;
			end
			if wordTable[1] == nil then
				returnDescription = returnDescription .. subDescription;
				break;
			end
			tempDescription = subDescription .. " " .. wordTable[1];
			if tempDescription:len() >= maxCharactersPerLine then
				returnDescription = returnDescription .. subDescription .. "\n";
				break;
			else
				subDescription = tempDescription;
				table.remove(wordTable, 1);
			end
		end
	end

	index = 1;
	--[[while index < description:len() do
		if description:sub(index, index+34):find("\n") == nil then -- make sure there isn't a new line character in the substring already
			tempString = description:sub(index, index+34):reverse(); -- reverse the string (we need to find the last instance of "space", this means the last instance of "space" will be the first one
			spaceIndex = tempString:find(" "); -- find the first instance of "space"
			if spaceIndex ~= nil then
				spaceIndex = 35 - spaceIndex; -- this finds the actual index in the non-reversed substring
				spaceIndex = spaceIndex + index - 1; -- this finds the actual index of the "space" in the full description string
				description = description:sub(0, spaceIndex) .. "\n" .. description:sub(spaceIndex+1);
				index = spaceIndex+2; -- move the index to right after the new \n
			else
				description = description:sub(0, index+34) .. "\n" .. description:sub(index+35);
				index = index + 1; -- move the index to right after the new \n
			end
		else
			index = (description:sub(index, index+34):find("\n")) + index; -- move the index to right after the existing \n
		end
	end]]--
	return returnDescription;
end

-- creates a tooltip hint.
function WEQuickslotWindow:CreateHint(gambit, control)

	local gambitName = gambit:GetName();
	local gambitDesc = gambit:GetDescription();

	gambitDesc = gambitDesc:gsub("<(.-)>", ""); -- remove color information from description

	gambitDesc = AddNewLineCharactersToDescription(gambitDesc);

	newLineCounter = 0;
	for index = 1, gambitDesc:len() do
		if gambitDesc:sub(index, index) == "\n" then
			newLineCounter = newLineCounter + 1;
		end
	end

	-- calculates a rough estimate of height needed
	-- Each line supports about 30 characters, round that result up (to make sure partial lines are added in), multiply by 18 (18 is the height of a line)
	-- For each new line character, add 18 to the height
	local tooltipHeight = (math.ceil((gambitDesc:len() / 33), 0) * 18) + (16 * newLineCounter);
	tooltipHeight = 20 * (newLineCounter+1);
	tooltipHeight = (14.3 * (newLineCounter+1)) + 14.8
	--Turbine.Shell.WriteLine(newLineCounter);

	local x, y = control:GetPosition(); -- get the top left corner of the control (the gambit icon)
	y = y + self.quickslotSize; -- we want the top left of the tooltip to be direct beneath the gambit icon
	
	x, y = control:GetParent():PointToScreen(x, y); -- convert the point to the screen coordinate

	local tooltipWidth = 250;

	window = Turbine.UI.Window();
	window:SetZOrder(250); -- make sure tooltip appears above everything else
	window:SetBackColor(Turbine.UI.Color(0,0,0));
	window:SetOpacity(.85);
	window:SetSize( tooltipWidth, tooltipHeight + 25); -- add 25 for the tooltip heading that contains the gambit name
	window:SetVisible( true );

	local displayWidth = Turbine.UI.Display:GetWidth();
	local displayHeight = Turbine.UI.Display:GetHeight();

	-- if the tooltip will go off the side of the screen, move it to the left enough so that it doesn't
	if x+tooltipWidth > displayWidth then
		x = displayWidth - tooltipWidth;
	end
	-- if the tooltip would go off the bottom of the screen, move it to above the gambit control
	if y+(tooltipHeight+25) > displayHeight then
		y = y - self.quickslotSize - (tooltipHeight+25);
	end

	window:SetPosition(x, y);

	lbl = Turbine.UI.Label();
	lbl:SetPosition(0,0);
	lbl:SetSize(tooltipWidth,25);
	lbl:SetFont(Turbine.UI.Lotro.Font.TrajanPro23);
	if gambitName:len() > 20 and gambitName:len() <= 25 then lbl:SetFont(Turbine.UI.Lotro.Font.TrajanPro18); -- in the case the gambit name is longer than 20 characters, reduce its size so it fits
	elseif gambitName:len() > 25 then lbl:SetFont(Turbine.UI.Lotro.Font.TrajanPro14); end
	lbl:SetText(gambitName);
	lbl:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter);
	lbl:SetForeColor(Turbine.UI.Color(1,1,1));
	lbl:SetMouseVisible(false);
	lbl:SetZOrder(3);
	lbl:SetBackColor(Turbine.UI.Color(0,0,0));
	lbl:SetOpacity(.85);
	lbl:SetParent(window);

	lbl = Turbine.UI.Label();
	lbl:SetPosition(10,25);
	lbl:SetSize(tooltipWidth-20,tooltipHeight); -- subtract 20 from the width to give a little space to the sides
	lbl:SetFont(Turbine.UI.Lotro.Font.Verdana14);
	lbl:SetText(gambitDesc);
	lbl:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	lbl:SetForeColor(Turbine.UI.Color(1,1,1));
	lbl:SetMouseVisible(false);
	lbl:SetMultiline(true);
	lbl:SetZOrder(3);
	lbl:SetBackColor(Turbine.UI.Color(0,0,0));
	lbl:SetOpacity(.85);
	lbl:SetParent(window);

	--[[ Below is the old code for tooltips. Will probably revert once better tooltips are implemented.
	local stance = player:GetClassAttributes():GetStance();
	local gambitName = gambit:GetName();

	-- set the height based on the height of the tooltip, to prevent the tooltip from "doubling"
	local tooltipHeight;
	
	if stance == 0 then -- no stance
		tooltipHeight = SDRPlugins.WardenEase2.GambitTooltips.NoStance.GambitHeights[gambitName];
	elseif stance == 1 then -- Recklessness stance
		tooltipHeight = SDRPlugins.WardenEase2.GambitTooltips.Recklessness.GambitHeights[gambitName];
	elseif stance == 2 then -- Determination stance
		tooltipHeight = SDRPlugins.WardenEase2.GambitTooltips.Determination.GambitHeights[gambitName];
	elseif stance == 3 then -- Assailment stance
		tooltipHeight = SDRPlugins.WardenEase2.GambitTooltips.Assailment.GambitHeights[gambitName];
	end
	-- to prevent the height from being nil, set it to 0.
	-- In this way if a tooltip height isn't entered, the tooltip won't display, thus allowing me to find tooltips that are broken
	if tooltipHeight == nil then
		tooltipHeight = 0;
	end

	local x, y = control:GetPosition(); -- get the top left corner of the control (the gambit icon)
	y = y + self.quickslotSize; -- we want the top left of the tooltip to be direct beneath the gambit icon
	
	x, y = control:GetParent():PointToScreen(x, y); -- convert the point to the screen coordinate

	local tooltipWidth = 235; -- roughly the size of a tooltip in LOTRO

	window = Turbine.UI.Window();
	window:SetZOrder(250); -- make sure tooltip appears above everything else
	window:SetSize( tooltipWidth, tooltipHeight );
	window:SetVisible( true );

	local displayWidth = Turbine.UI.Display:GetWidth();
	local displayHeight = Turbine.UI.Display:GetHeight();

	-- if the tooltip will go off the side of the screen, move it to the left enough so that it doesn't
	if x+tooltipWidth > displayWidth then
		x = displayWidth - tooltipWidth;
	end
	-- if the tooltip would go off the bottom of the screen, move it to above the gambit control
	if y+tooltipHeight > displayHeight then
		y = y - self.quickslotSize - tooltipHeight;
	end

	window:SetPosition(x, y);
	
	-- if the height is zero, I haven't created the tooltip for that gambit yet, only set backgrounds for created tooltips
	if tooltipHeight ~= 0 then
		if stance == 0 then -- no stance
			window:SetBackground("SDRPlugins/WardenEase2/GambitTooltips/NoStance/"..gambit:GetName()..".jpg");
		elseif stance == 1 then -- Recklessness stance
			window:SetBackground("SDRPlugins/WardenEase2/GambitTooltips/Recklessness/"..gambit:GetName()..".jpg");
		elseif stance == 2 then -- Determination stance
			window:SetBackground("SDRPlugins/WardenEase2/GambitTooltips/Determination/"..gambit:GetName()..".jpg");
		elseif stance == 3 then -- Assailment stance
			window:SetBackground("SDRPlugins/WardenEase2/GambitTooltips/Assailment/"..gambit:GetName()..".jpg");
		end
	end --]]--

	return window;
end

-- debugging purposes
function WEQuickslotWindow:pvar(tmp)
	Turbine.Shell.WriteLine(tmp);
end

-- refreshes the layout of the bar
function WEQuickslotWindow:RefreshLayout()
	local displayWidth = Turbine.UI.Display:GetWidth();
	local displayHeight = Turbine.UI.Display:GetHeight();

	-- default is to have the bar vertical
	self.slotsWide = 1;
	self.slotsHigh = #(self.barGambits);

	-- if the bar is switched, switch slotsWide and slotsHigh
	if self.settings.transposed then
		self.slotsWide, self.slotsHigh = self.slotsHigh, self.slotsWide;
	end

	local i, width, height;

	-- width of the bar which is
	-- number of quickslots vertical * the spacing for each, minus the extra spacing at the end
	width = self.slotsWide * (self.spacing + self.quickslotSize) - self.spacing;
	
	-- height of the bar which is
	-- number of quickslots horizontal times the spacing for each quickslot
	height = self.slotsHigh * (self.spacing + self.quickslotSize);

	self:SetSize(width, height)

	for i = 1, #(self.barGambits), 1 do
		local col, row;

		-- set the positions of the controls
		col = (i - 1) % self.slotsWide;
		row = math.floor((i - 1) / self.slotsWide);
		self.gambitControls[i]:SetPosition( col * (self.quickslotSize + self.spacing), row * (self.quickslotSize + self.spacing) );
	end
end

-- -- function WEQuickslotWindow:GetBarGambits()